#
#++
#
#  Copyright Digital Equipment Corporation 1997. All Rights Reserved.
#
#  Restricted Rights: Use, duplication, or disclosure by the U.S. Government
#  is subject to restrictions as set forth in subparagraph (c)(1)(ii) of DFARS
#  252.227-7013, or in FAR 52.227-19, or in FAR 52.227-14 Alt. III, as
#  applicable.
#
#  This software is proprietary to and embodies the confidential technology of
#  Digital Equipment Corporation. Possession, use, or copying of this software
#  and media is authorized only pursuant to a valid written license from
#  Digital or an authorized sublicensor.
#
#--
#
#**++
#
#  MODULE:
#
#      D_FDDI.DDL
#
#  ABSTRACT:
#
#      DDL program to DECODE FDDI packets.
#
#  AUTHORS:
#
#      A A Hart
#
#  DATE:
#
#      6 Jun 1997
#
#  MODIFICATION HISTORY:
#
INCLUDE ddlinc.ddl


# CONSTANTS
#
CONSTANT
	FDDI_frame_control               = 0,
	FDDI_da                          = 1,
	FDDI_sa                          = 7,
	FDDI_pid                         = 13,
	FDDI_sizeof                      = 21;

CONSTANT
	CRC_PRESENT			= 4,
	CRC_NOT_PRESENT			= 0;



# DECODE: d_fddi
# The $2 variable determines whether the CRC is present, if so the length of
# packet is consequently reduced by 4.
#
# $1 = REVERSE or DONT_REVERSE
# $2 = CRC_PRESENT OR CRC_NOT_PRESENT
# CO = FDDI header.
#
DECLARE PST pst_8022;
DECODE d_fddi(0) {
	TITLE = "FDDI",
	PLEVEL = HL_FDDI,
	NEXT = pst_8022,
	CONSUME = FDDI_pid
	} =
		hex( byte(FDDI_frame_control), 2, "Frame control" ),
		mac_name( FDDI_da, $1, "Destination MAC address" ) { BRIEF },
		mac_name( FDDI_sa, $1, "Source MAC address" ) { BRIEF },
		$3 = remaining(),
		$3 = $3 - $2,
		length( $3, 0, "" )
		;



# PST: pst_fddi
# Main entry point for packets with natural (i.e. unreversed) MAC addresses.
# Unconditionally passes control to the d_fddi_r decode (assumes that the
# CRC is present in the packet).
#
# CO = start of FDDI packet.
#
DECODE d_fddi_r( 0 ) { NEXT = d_fddi } = $1 = REVERSE, $2 = CRC_PRESENT;
PST pst_fddi( 0 ) {
	TITLE = "FDDI packet",
	TYPE = "FDDI",
	DEFAULT = d_fddi
	} = ;



# PST: pst_fddi_canonical
# Main entry point for packets with previously reversed MAC addresses.
# Unconditionally passes control to the d_fddi_nr decode (assumes that the
# CRC is not present in the packet).
#
# CO = start of FDDI packet.
#
DECODE d_fddi_nr( 0 ) { NEXT = d_fddi } = $1 = DONT_REVERSE, $2 = CRC_NOT_PRESENT;
PST pst_fddi_canonical( 0 ) {
	TITLE = "FDDI packet",
	TYPE = "FDDIcanonical",
	DEFAULT = d_fddi_nr
	} = ;

