#
#++
#
#  Copyright Digital Equipment Corporation 1997. All Rights Reserved.
#
#  Restricted Rights: Use, duplication, or disclosure by the U.S. Government
#  is subject to restrictions as set forth in subparagraph (c)(1)(ii) of DFARS
#  252.227-7013, or in FAR 52.227-19, or in FAR 52.227-14 Alt. III, as
#  applicable.
#
#  This software is proprietary to and embodies the confidential technology of
#  Digital Equipment Corporation. Possession, use, or copying of this software
#  and media is authorized only pursuant to a valid written license from
#  Digital or an authorized sublicensor.
#
#--
#
#++
#
#  MODULE:
#
#      D_PPTP.DDL
#
#  ABSTRACT:
#
#	DDL program to decode PPTP (point to point tunnelling protocol) packets.
#
#  AUTHORS:
#
#      John Rigby
#
#  DATE:
#
#      22 Dec 1997
#
#  MODIFICATION HISTORY:
#
INCLUDE ddlinc.ddl
DECLARE PST pst_tcpd,pst_tcps; #to ATTACH to

# CONSTANTs
#
#      0                   1                   2                   3
#      0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |		   Length	      |       PPTP Message Type       |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                         Magic Cookie                          |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |     Control Message Type      |           Reserved0           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |       Protocol Version        |           Reserved1           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                     Framing Capabilities                      |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                      Bearer Capabilities                      |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |       Maximum Channels        |       Firmware Revision       |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                                                               |
#     +                     Host Name (64 octets)                     +
#     |                                                               |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                                                               |
#     +                   Vendor String (64 octets)                   +
#     |                                                               |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+



CONSTANT
	PPTP_length			= 0,
	PPTP_message_type		= PPTP_length+2,
	PPTP_cookie			= PPTP_message_type+2,
	PPTP_control_message_type	= PPTP_cookie+4,
	PPTP_reserved0			= PPTP_control_message_type+2,
	PPTP_protocol_version		= PPTP_reserved0+2,
	PPTP_result_code		= PPTP_protocol_version+2,
	PPTP_error_code			= PPTP_result_code + 1,
	PPTP_reserved1			= PPTP_protocol_version+2,
	PPTP_framing_capabilities	= PPTP_reserved1+2,
	PPTP_bearer_capabilities	= PPTP_framing_capabilities+4,
	PPTP_maximum_channels		= PPTP_bearer_capabilities+4,
	PPTP_firmware_revision		= PPTP_maximum_channels+2,
	PPTP_host_name			= PPTP_firmware_revision+2,
	PPTP_vendor_string		= PPTP_host_name+64
	;

#      0                   1                   2                   3
#      0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |            Length             |       PPTP Message Type       |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                         Magic Cookie                          |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |     Control Message Type      |           Reserved0           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |            Call ID            |      Call Serial Number       |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                          Minimum BPS                          |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                          Maximum BPS                          |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                          Bearer Type                          |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                         Framing Type                          |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |   Packet Recv. Window Size    |    Packet Processing Delay    |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |      Phone Number Length      |           Reserved1           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                                                               |
#     +                   Phone Number (64 octets)                    +
#     |                                                               |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                                                               |
#     +                    Subaddress (64 octets)                     +
#     |                                                               |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

CONSTANT
	PPTP_call_id		= PPTP_reserved0+2,
	PPTP_call_serial_number = PPTP_call_id+2,
	PPTP_minimum_bps	= PPTP_call_serial_number+2,
	PPTP_maximum_bps	= PPTP_minimum_bps+4,
	PPTP_bearer_type	= PPTP_maximum_bps+4,
	PPTP_framing_type	= PPTP_bearer_type+4,
	PPTP_rcv_window_size	= PPTP_framing_type+4,
	PPTP_packet_delay	= PPTP_rcv_window_size+2,
	PPTP_phone_number_length= PPTP_packet_delay+2,
	PPTP_phone_number	= PPTP_phone_number_length+2+2,
	PPTP_subaddress		= PPTP_phone_number+64
	;

#      0                   1                   2                   3
#      0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |            Length             |      PPTP Message Type        |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                         Magic Cookie                          |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |     Control Message Type      |           Reserved0           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |            Call ID            |       Peer's Call ID          |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |  Result Code  |  Error Code   |          Cause Code           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                         Connect Speed                         |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |   Packet Recv. Window Size    |    Packet Processing Delay    |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                      Physical Channel ID                      |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
CONSTANT
	PPTP_peer_call_ID	= PPTP_call_id+2,
	PPTP_ocr_result_code	= PPTP_peer_call_ID+2,
	PPTP_ocr_error_code	= PPTP_ocr_result_code+1,
	PPTP_cause_code		= PPTP_ocr_error_code+1,
	PPTP_connect_speed	= PPTP_cause_code+2,
	PPTP_ocr_rcv_window_size= PPTP_connect_speed+4,
	PPTP_ocr_packet_delay	= PPTP_ocr_rcv_window_size+2,
	PPTP_physical_channel	= PPTP_ocr_packet_delay+2
	;
#set-link-info
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |            Length             |      PPTP Message Type        |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                         Magic Cookie                          |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |     Control Message type      |           Reserved0           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |        Peer's Call ID         |           Reserved1           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                           Send ACCM                           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                         Receive ACCM                          |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

CONSTANT
	PPTP_tx_ACCM	= PPTP_call_id+4,
	PPTP_rx_ACCM	= PPTP_tx_ACCM+4;
#stop-control-connection
#      0                   1                   2                   3
#      0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |		   Length	      |       PPTP Message Type       |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                         Magic Cookie                          |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |     Control Message Type      |           Reserved0           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |    Reason     |   Reserved1   |           Reserved2           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#
CONSTANT
	PPTP_scc_reason = PPTP_reserved0+2,
	PPTP_scc_error	= PPTP_scc_reason+2;

#call-disconnect-notify
#      0                   1                   2                   3
#      0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |            Length             |      PPTP Message Type        |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                         Magic Cookie                          |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |     Control Message Type      |           Reserved0           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |            Call ID            |  Result Code  |  Error Code   |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |          Cause Code           |           Reserved1           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                                                               |
#     +              Call Statistics (128 octets)                     +
#     |                                                               |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
CONSTANT
	PPTP_cdn_result_code	= PPTP_call_id+2,
	PPTP_cdn_error_code	= PPTP_cdn_result_code+1,
	PPTP_cdn_cause_code	= PPTP_cdn_error_code+1,
	PPTP_call_statistics	= PPTP_cdn_cause_code+4,
	;
#Echo-request
#      0                   1                   2                   3
#      0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |            Length             |       PPTP Message Type       |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                         Magic Cookie                          |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |     Control Message Type      |           Reserved0           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                          Identifier                           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+ 
#     |  Result Code  |   Error Code  |           Reserved1           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
CONSTANT
	PPTP_identifier = PPTP_reserved0+2,
	PPTP_er_result  = PPTP_identifier+4,
	PPTP_er_error   = PPTP_er_result+1;
#Incoming-Call-Request
#      0                   1                   2                   3
#      0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |            Length             |       PPTP Message Type       |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                         Magic Cookie                          |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |     Control Message Type      |           Reserved0           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |            Call ID            |      Call Serial Number       |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                       Call Bearer Type                        |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                      Physical Channel ID                      |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |     Dialed Number Length      |     Dialing Number Length     |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                                                               |
#     +                   Dialed Number (64 octets)                   +
#     |                                                               |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                                                               |
#     +                  Dialing Number (64 octets)                   +
#     |                                                               |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                                                               |
#     +                    Subaddress (64 octets)                     +
#     |                                                               |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
CONSTANT
	PPTP_icr_bearer_type = PPTP_call_serial_number+2,
	PPTP_icr_physical_channel = PPTP_icr_bearer_type+4,
	PPTP_icr_dialed_length = PPTP_icr_physical_channel+4,
	PPTP_icr_dialing_length = PPTP_icr_dialed_length+2,
	PPTP_icr_dialed = PPTP_icr_dialing_length+2,
	PPTP_icr_dialing = PPTP_icr_dialed+64,
	PPTP_icr_subaddress = PPTP_icr_dialing+64;
#Incoming-Call-Reply
#      0                   1                   2                   3
#      0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |            Length             |       PPTP Message Type       |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                         Magic Cookie                          |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |     Control Message Type      |           Reserved0           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |            Call ID            |       Peer's Call ID          |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |  Result Code  |  Error Code   |   Packet Recv. Window Size    |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |     Packet Transmit Delay     |           Reserved1           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
CONSTANT
	PPTP_icrep_result_code		= PPTP_peer_call_ID+2,
	PPTP_icrep_error_code		= PPTP_icrep_result_code+1,
	PPTP_icrep_rcv_window_size	= PPTP_icrep_error_code+1,
	PPTP_icrep_packet_delay		= PPTP_icrep_rcv_window_size+2
	;
#Incoming-Call-Connected
#      0                   1                   2                   3
#      0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |            Length             |      PPTP Message Type        |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                         Magic Cookie                          |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |     Control Message Type      |           Reserved0           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |       Peer's Call ID          |           Reserved1           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                         Connect Speed                         |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |   Packet Recv. Window Size    |     Packet Transmit Delay     |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                         Framing Type                          |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
CONSTANT
	PPTP_icc_connect_speed	= PPTP_call_id+4,
	PPTP_icc_rcv_window_size= PPTP_icc_connect_speed+4,
	PPTP_icc_packet_delay	= PPTP_icc_rcv_window_size+2,
	PPTP_icc_framing_type	= PPTP_icc_packet_delay+2
	;
#WAN-Error-Notify
#      0                   1                   2                   3
#      0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |            Length             |      PPTP Message Type        |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                         Magic Cookie                          |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |     Control Message type      |           Reserved0           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |        Peer's Call ID         |           Reserved1           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                          CRC Errors                           |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                        Framing Errors                         |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                       Hardware Overruns                       |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                        Buffer Overruns                        |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                        Time-out Errors                        |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#     |                       Alignment Errors                        |
#     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
CONSTANT
	PPTP_crc_errors = PPTP_call_id+4,
	PPTP_framing_errors = PPTP_crc_errors+4,
	PPTP_hardware_overruns = PPTP_framing_errors+4,
	PPTP_buffer_overruns = PPTP_hardware_overruns+4,
	PPTP_timeout_errors = PPTP_buffer_overruns+4,
	PPTP_alignment_errors = PPTP_timeout_errors+4;

FLAG_TABLE pptp_fc_flags  {
    TITLE = "Framing Capabilities"
    } = 
	0x01 = "Async",
	0x02 = "Sync"
	;
FLAG_TABLE pptp_bc_flags  {
    TITLE = "Bearer Capabilities"
    } = 
	0x01 = "Analog",
	0x02 = "Digital"
	;

DECODE 
	Start_Control_Connection_Request( 1 )
		{TITLE		= "Start Control Connection Request"
		,CONSUME	= remaining()
		,BRIEF_TITLE	= "StaReq"
		}=
	length ( uword(PPTP_length), 0, "PPTP length" ) {BRIEF},
	hex ( long(PPTP_cookie), 8, "Magic Cookie" ),
	print( "Version", "V%d.%d", byte (PPTP_protocol_version), byte (PPTP_protocol_version+1)),
	flags( long(PPTP_framing_capabilities), 0, pptp_fc_flags ),
	flags( long(PPTP_bearer_capabilities), 0, pptp_bc_flags ),
	decimal (word(PPTP_maximum_channels),0,"Maximum Channels"),
	print( "Firmware Revision", "V%d", word (PPTP_firmware_revision)),
	ascii_dump_string(PPTP_host_name,0,"Host Name"),
	ascii_dump_string(PPTP_vendor_string,0,"Vendor String")
	;
NAME_LIST pptp_result_codes {
	TITLE = "Result"
	} =
	"",				#0
	"Successful establishment",	#1
	"General Error",		#2
	"Command channel exists",	#3
	"Requester is not authorized",	#4
	"Version not supported",	#5
	;
NAME_LIST pptp_error_codes {
	TITLE = "Error"
	} =
	"None",		#0
	"Not Connected",#1
	"Bad Format",	#2
	"Bad Value",	#3
	"No Resource",	#4
	"Bad Call ID",	#5
	"PAC error",	#6
	;

DECODE 
	Start_Control_Connection_Reply( 2 )
		{TITLE = "Start Control Connection Reply"
		, CONSUME = remaining()
		,BRIEF_TITLE	= "StaRep"
		}=
	length ( uword(PPTP_length), 0, "PPTP length" ) {BRIEF},
	hex ( long(PPTP_cookie), 8, "Magic Cookie" ),
	print( "Version", "V%d.%d", byte (PPTP_protocol_version), byte (PPTP_protocol_version+1)),
	list( byte( PPTP_result_code), 0, pptp_result_codes){BRIEF},
	list( byte( PPTP_error_code), 0, pptp_error_codes),
	
	flags( long(PPTP_framing_capabilities), 0, pptp_fc_flags ),
	flags( long(PPTP_bearer_capabilities), 0, pptp_bc_flags ),
	decimal (word(PPTP_maximum_channels),0,"Maximum Channels"),
	print( "Firmware Revision", "V%d", word (PPTP_firmware_revision)),
	ascii_dump_string(PPTP_host_name,0,"Host Name"),
	ascii_dump_string(PPTP_vendor_string,0,"Vendor String")
	;
NAME_LIST pptp_scc_reasons {
	TITLE = "Reason"
	} =
	"",				#0
	"General",			#1
	"Stop Protocol",		#2
	"Shutdown",			#3
	;
NAME_LIST pptp_scc_results { #also used for echo request results
	TITLE = "Results"
	} =
	"",			#0
	"OK",			#1
	"General Error",	#2
	;
DECODE 
	Stop_Control_Connection_Request( 3 )
		{CONSUME = remaining()
		,TITLE="Stop Control Connection Request"
		,BRIEF_TITLE	= "StoReq"
		}=
	length ( uword(PPTP_length), 0, "PPTP length" ) {BRIEF},
	hex ( long(PPTP_cookie), 8, "Magic Cookie" ),
	list ( byte ( PPTP_scc_reason),0,pptp_scc_reasons){BRIEF},
	;

DECODE 
	Stop_Control_Connection_Reply( 4 )
		{CONSUME	= remaining()
		,TITLE		="Stop Control Connection Reply"
		,BRIEF_TITLE	= "StoRep"
		}=
	length ( uword(PPTP_length), 0, "PPTP length" ) {BRIEF},
	hex ( long(PPTP_cookie), 8, "Magic Cookie" ),
	list ( byte ( PPTP_scc_reason),0,pptp_scc_results){BRIEF},
	list( byte( PPTP_scc_error), 0, pptp_error_codes),
	;
DECODE 
	Echo_Request( 5 )
		{CONSUME = remaining()
		,TITLE="Echo Request"
		,BRIEF_TITLE="EcoReq"
		}=
	length ( uword(PPTP_length), 0, "PPTP length" ) {BRIEF},
	hex ( long(PPTP_cookie), 8, "Magic Cookie" ),
	hex ( long(PPTP_identifier), 8, "Identifier" ){BRIEF};
DECODE 
	Echo_Reply( 6 )
		{CONSUME = remaining()
		,TITLE="Echo Reply"
		,BRIEF_TITLE="EcoRep"
		}=
	length ( uword(PPTP_length), 0, "PPTP length" ) {BRIEF},
	hex ( long(PPTP_cookie), 8, "Magic Cookie" ),
	hex ( long(PPTP_identifier), 8, "Identifier" ){BRIEF},
	list ( byte ( PPTP_er_result ),0,pptp_scc_results){BRIEF},
	list( byte( PPTP_er_error ), 0, pptp_error_codes);
DECODE 
	Outgoing_Call_Request( 7 )
		{CONSUME	= remaining()
		,TITLE		="Outgoing Call Request"
		,BRIEF_TITLE	= "OCReq"
		}=
	length ( uword(PPTP_length), 0, "PPTP length" ) {BRIEF},
	hex ( long(PPTP_cookie), 8, "Magic Cookie" ),
	decimal (word (PPTP_call_id),0,"Call ID"){BRIEF},
	decimal (word (PPTP_call_serial_number),0,"Call Serial Number"),
	decimal (long (PPTP_minimum_bps),0,"Minimum bps"),
	decimal (long (PPTP_maximum_bps),0,"Maximum bps"),
	flags( long(PPTP_bearer_type), 0, pptp_bc_flags ),
	flags( long(PPTP_framing_type), 0, pptp_fc_flags ),
	decimal (word (PPTP_rcv_window_size),0,"Rx Window Size"),
	decimal (word (PPTP_packet_delay),0,"Packet delay"),
	ascii_dump_string(PPTP_phone_number,0,"Phone Number"){BRIEF},
	ascii_dump_string(PPTP_subaddress,0,"Subaddress")
	;
NAME_LIST pptp_ocr_result_codes {
	TITLE = "Result"
	} =
	"",				#0
	"Successful establishment",	#1
	"General Error",		#2
	"No Carrier",			#3
	"Busy",				#4
	"No Dial Tone",			#5
	"Timeout",			#6
	"Administratively Prohibited"	#7
	;
DECODE 
	Outgoing_Call_Reply( 8 )
		{CONSUME	= remaining()
		,TITLE		="Outgoing Call Reply"
		,BRIEF_TITLE	= "OCRep"
		}=
	length ( uword(PPTP_length), 0, "PPTP length" ) {BRIEF},
	hex ( long(PPTP_cookie), 8, "Magic Cookie" ),
	decimal (word (PPTP_call_id),0,"Call ID"),
	decimal (word (PPTP_peer_call_ID),0,"Peer's Call ID"),
	list( byte( PPTP_ocr_result_code), 0, pptp_ocr_result_codes){BRIEF},
	list( byte( PPTP_ocr_error_code), 0, pptp_error_codes),
	hex ( word(PPTP_cause_code),4,"Cause code"),
	decimal (long(PPTP_connect_speed),0,"Connect Speed"),
	decimal (word (PPTP_ocr_rcv_window_size),0,"Rx Window Size"),
	decimal (word (PPTP_ocr_packet_delay),0,"Packet delay"),
	hex (long (PPTP_physical_channel),8,"Physical channel ID")
	;
DECODE 
	Incoming_Call_Request( 9 )
		{CONSUME = remaining()
		,TITLE="Incoming Call Request"
		,BRIEF_TITLE="ICReq"
		}=
	length ( uword(PPTP_length), 0, "PPTP length" ) {BRIEF},
	hex ( long(PPTP_cookie), 8, "Magic Cookie" ),
	decimal (word (PPTP_call_id),0,"Call ID"){BRIEF},
	decimal (word (PPTP_call_serial_number),0,"Call Serial Number"),
	flags( long(PPTP_icr_bearer_type), 0, pptp_bc_flags ),
	hex (long (PPTP_icr_physical_channel),8,"Physical channel ID"),
	ascii_dump_string(PPTP_icr_dialed,0,"Dialed Number"){BRIEF},
	ascii_dump_string(PPTP_icr_dialing,0,"Dialing Number"),
	ascii_dump_string(PPTP_icr_subaddress,0,"Subaddress")
	;

NAME_LIST pptp_icrep_result_codes {
	TITLE = "Result"
	} =
	"",			#0
	"Connect",		#1
	"General Error",	#2
	"Do Not Accept";	#3

DECODE Incoming_Call_Reply( 10 ){CONSUME = remaining(),TITLE="Incoming Call Reply"}=
	length ( uword(PPTP_length), 0, "PPTP length" ) {BRIEF},
	hex ( long(PPTP_cookie), 8, "Magic Cookie" ),
	decimal (word (PPTP_call_id),0,"Call ID"),
	decimal (word (PPTP_peer_call_ID),0,"Peer's Call ID"),
	list( byte( PPTP_icrep_result_code), 0, pptp_icrep_result_codes){BRIEF},
	list( byte( PPTP_icrep_error_code), 0, pptp_error_codes),
	decimal (word (PPTP_icrep_rcv_window_size),0,"Rx Window Size"),
	decimal (word (PPTP_icrep_packet_delay),0,"Packet delay");
DECODE 
	Incoming_Call_Connected( 11 )
		{CONSUME = remaining()
		,TITLE="Incoming Call Connected"
		,BRIEF_TITLE = "ICConn"
		}=
	length ( uword(PPTP_length), 0, "PPTP length" ) {BRIEF},
	hex ( long(PPTP_cookie), 8, "Magic Cookie" ),
	decimal (word (PPTP_call_id),0,"Peer Call ID"),
	decimal (long(PPTP_icc_connect_speed),0,"Connect Speed"),
	decimal (word (PPTP_icc_rcv_window_size),0,"Rx Window Size"),
	decimal (word (PPTP_icc_packet_delay),0,"Packet delay"),
	flags( long(PPTP_icc_framing_type), 0, pptp_fc_flags );
	
DECODE 
	Call_Clear_Request( 12 )
		{CONSUME	= remaining()
		,TITLE		="Call Clear Request"
		,BRIEF_TITLE	= "ClrReq"
		}=
	length ( uword(PPTP_length), 0, "PPTP length" ) {BRIEF},
	hex ( long(PPTP_cookie), 8, "Magic Cookie" ),
	decimal (word (PPTP_call_id),0,"Call ID"){BRIEF},
	;
NAME_LIST pptp_cdn_result_codes {
	TITLE = "Result"
	} =
	"",				#0
	"Lost Carrier",#1
	"General Error",		#2
	"Administrative",#3
	"received Call-Clear-Request",#4
	;
DECODE 
	Call_Disconnect_Notify( 13 )
		{CONSUME	= remaining()
		,TITLE		="Call Disconnect Notify"
		,BRIEF_TITLE	= "CalDis"
		}=
	length ( uword(PPTP_length), 0, "PPTP length" ) {BRIEF},
	hex ( long(PPTP_cookie), 8, "Magic Cookie" ),
	decimal (word (PPTP_call_id),0,"Call ID"),
	list( byte( PPTP_cdn_result_code), 0, pptp_cdn_result_codes){BRIEF},
	list( byte( PPTP_cdn_error_code), 0, pptp_error_codes),
	hex ( word(PPTP_cdn_cause_code),4,"Cause code"),
	ascii_dump_string( PPTP_call_statistics,0,"Call Statistics")
	;
DECODE 
	WAN_Error_Notify( 14 )
		{CONSUME = remaining()
		,TITLE="WAN Error Notify"
		,BRIEF_TITLE="WANErr"
		}=
	length ( uword(PPTP_length), 0, "PPTP length" ) {BRIEF},
	hex ( long(PPTP_cookie), 8, "Magic Cookie" ),
	decimal (word (PPTP_call_id),0,"Peer's Call ID"),
	decimal (long (PPTP_framing_errors),0,"Framing Errors"),
	decimal (long (PPTP_hardware_overruns ),0,"Hardware Overruns"),
	decimal (long (PPTP_buffer_overruns ),0,"Buffer Overruns"),
	decimal (long (PPTP_timeout_errors ),0,"Timeout Errors"),
	decimal (long (PPTP_alignment_errors ),0,"Alignment Errors")
;
DECODE 
	Set_Link_Info( 15 )
		{CONSUME	=remaining()
		,TITLE		="Set Link Info"
		,BRIEF_TITLE	="SetLnk"
		}= 
	length ( uword(PPTP_length), 0, "PPTP length" ) {BRIEF},
	hex ( long(PPTP_cookie), 8, "Magic Cookie" ),
	decimal (word (PPTP_call_id),0,"Peer Call ID"),
	hex (long (PPTP_tx_ACCM),8,"Send ACCM"),
	hex (long (PPTP_rx_ACCM),8,"Receive ACCM")
	;

PST pptp_messages ( word(PPTP_control_message_type) ) { TITLE = "PPTP message" } =
	   Start_Control_Connection_Request,
           Start_Control_Connection_Reply,
           Stop_Control_Connection_Request,
	   Stop_Control_Connection_Reply,
	   Echo_Request,
	   Echo_Reply,
	   Outgoing_Call_Request,
	   Outgoing_Call_Reply,
	   Incoming_Call_Request,
	   Incoming_Call_Reply,
	   Incoming_Call_Connected,
	   Call_Clear_Request,
	   Call_Disconnect_Notify,
	   WAN_Error_Notify,
	   Set_Link_Info;

DECODE d_pptps(1723) {CONSUME = $1, BRIEF_TITLE="PPTP", ATTACH = pst_tcps, NEXT = pptp_messages} = ;
DECODE d_pptpd(1723) {CONSUME = $1, BRIEF_TITLE="PPTP", ATTACH = pst_tcpd, NEXT = pptp_messages} = ;
